﻿XIncludeFile "Object.pbi"
Macro ASSERT(a, b, e="")
	If (a <> b)
		Debug "Assert Error: " + e
	EndIf
EndMacro
Macro ASSERT_NOT(a, b, e="")
	If (a = b)
		Debug "Assert Error: " + e
	EndIf
EndMacro

Class(test1)
EndClass

Class(test2)
	Members
		a.i
EndClass

Class(test3)
	StaticMembers
		Define version.d = 3.00
EndClass
ASSERT(test3::version, 3.00, "Wrong version!")

Class(test4)
	Members
		a.i
	StaticMembers
		Define version.d = 4.00
EndClass
ASSERT(test4::version, 4.00, "Wrong version!")

Class(test5)
	StaticMembers
		Declare.d version()
	Members
		a.i
	Definition
		Procedure.d version()
			ProcedureReturn 5.00
		EndProcedure
EndClass
ASSERT(test5::version(), 5.00, "Wrong version!")

Class(test6)
	Constructors
EndClass
Class(test7)
	StaticMembers
	Members
	Constructors
EndClass
Class(test8)
	Members
	StaticMembers
	Constructors
EndClass
Class(test9)
	Members
	Constructors
EndClass
Class(test10)
	StaticMembers
	Constructors
EndClass

Class(test11)
	getA.i()
	Members
		a.i
	Constructors
		Construct(setA,(a.i))
	Definition
		Constructor(setA,(a.i))
			this\a = a
		EndConstructor
		
		Method(i,getA,(this))
			MethodReturn(this\a)
		EndMethod
EndClass
Define test11.type(test11) = test11::setA(123)
ASSERT_NOT(test11, 0, "test11::setA(123) not correctly initialized!")
ASSERT(test11\getA(), 123, "test11::getA() returns wrong number!")
test11\free()

Class(test12)
	getA.i()
	Members
		a.i
	Constructors
		Construct(setA,(a.i))
	Definition
		DefaultConstructor
			; call other constructor within default constructor.
			CallConstructor(setA,(5))
		EndDefaultConstructor
		
		Constructor(setA,(a.i))
			this\a = a
		EndConstructor
		
		Method(i,getA,(this))
			MethodReturn(this\a)
		EndMethod
EndClass
Define test12.type(test12) = new(test12)
ASSERT_NOT(test12, 0, "new(test12) not correctly initialized!")
ASSERT(test12\getA(), 5, "test12\getA() returned wrong number!")
test12\free()

Class(test13)
	Constructors
		Construct(Fail1,(a.i))
		Construct(Fail2,(a.i))
		Construct(Fail3,(a.i))
		Construct(Fail4,(a.i))
	Definition
		DefaultConstructor
			ConstructorFailure
		EndDefaultConstructor
		
		Constructor(Fail1,(a.i))
			ConstructorFailure
		EndConstructor
		
		Constructor(Fail2,(a.i))
			CallConstructor()
		EndConstructor
		
		Constructor(Fail3,(a.i))
			CallConstructor(Fail1,(a))
		EndConstructor
		
		Constructor(Fail4,(a.i))
			CallConstructor(Fail2,(a))
		EndConstructor
EndClass

ASSERT(new(test13), 0, "new(test13) should be 0!")
ASSERT(test13::Fail1(1), 0, "test13::Fail1(1) should be 0!")
ASSERT(test13::Fail2(1), 0, "test13::Fail2(1) should be 0!")
ASSERT(test13::Fail3(1), 0, "test13::Fail3(1) should be 0!")
ASSERT(test13::Fail4(1), 0, "test13::Fail4(1) should be 0!")

Class(test14)
EndClass

ASSERT(test14::__default_constructor__, 0, "new(test14) should fail!")

Class(test15)
	Definition
		PrivateDefaultConstructor
		EndDefaultConstructor
EndClass

ASSERT(test15::__default_constructor__, 0, "new(test15) should fail!")

Class(test16a)
	get1.i()
	get2.i()
	Members
		a.i
	Definition
		; Make Default Constructor invisible
		PrivateDefaultConstructor
			this\a = 100
		EndDefaultConstructor
		
		Method(i,get1,(this))
			MethodReturn(this\a)
		EndMethod
EndClass
ASSERT_NOT(callable(test16a,get1),0,"test16a\get1() should be callable!")
ASSERT(callable(test16a,get2),0,"test16a\get2() should not be callable!")

Class(test16b,test16a)
	get103.i()
	Definition
		; Make Default Constructor visible
		DefaultConstructor
		EndDefaultConstructor
		
		; Override get1()
		Method(i,get1,(this))
			MethodReturn(1)
		EndMethod
		
		;Implement get2()
		Method(i,get2,(this))
			MethodReturn(self\get1() + 1)
		EndMethod
		
		;Use get1() from this class and from base class
		Method(i,get103,(this))
			MethodReturn(super()\get1() + self\get1() + self\get2())
		EndMethod
EndClass

Define test16b.type(test16b) = new(test16b)
ASSERT(test16b\get1(), 1, "test16b\get1() should return 100.")
ASSERT(test16b\get2(), 2, "test16b\get2() should return 2.")
ASSERT(test16b\get103(), 103, "test16b\get3() should return 3.")
test16b\free()

Class(test17a)
	Definition
		DefaultConstructor
		EndDefaultConstructor
EndClass
Class(test17b, test17a)
	Definition
		DefaultConstructor
		EndDefaultConstructor
EndClass

Define test17a.type(test17a) = new(test17a)
Define test17b.type(test17b) = new(test17b)
ASSERT(isinstance(test17b,test17b), #True, "isinstance(test17b,test17b) should be true")
ASSERT(isinstance(test17b,test17a), #True, "isinstance(test17b,test17a) should be true")
ASSERT(isinstance(test17b,Object), #True, "isinstance(test17b,Object) should be true")
ASSERT(isinstance(test17a,test17b), #False, "isinstance(test17a,test17b) should be false")
ASSERT(isinstance(test17a,test17a), #True, "isinstance(test17a,test17a) should be true")
ASSERT(isinstance(test17a,Object), #True, "isinstance(test17a,Object) should be true")
; IDE Options = PureBasic 5.40 LTS Beta 9 (Linux - x64)
; CursorPosition = 36
; FirstLine = 17
; Folding = -----
; EnableUnicode
; EnableXP
; EnableOnError
; CompileSourceDirectory
; EnablePurifier