﻿XIncludeFile "InputStream.pbi"

; See: http://docs.oracle.com/javase/7/docs/api/java/io/FileInputStream.html
Class(FileInputStream, InputStream)
		; Closes the file.
		;close.i()
		
		; bytes available for reading
		;available.q()
		
		; reads size byes from the stream into *buffer.
		; returns bytes read or -1 if end of file was reached.
		;readBuffer.i(*buffer, size.i)
		
		; skips a certain amount of bytes.
		;skip.q(bytes.q)
		
		; marks a position in the stream to which can be jumped with reset
		; initially 0. readLimit is ignored.
		;mark(readLimit.q)
		
		; returns true
		;markSupported.i()
		
		; jumps to the marked position.
		;reset()
		
		; returns the file id.
		getId.i()
	Members
		fileId.i
		marked.q
	Constructors
		Construct(open,(file.s))
		Construct(useFile,(fileId.i))
	Definition
		PrivateDefaultConstructor
			this\marked = 0
		EndDefaultConstructor
		
		Constructor(open, (file.s))
			Protected fileId.i = ReadFile(#PB_Any, file)
			CallConstructor(useFile,(fileId))
		EndConstructor
		
		Constructor(useFile, (fileId.i))
			CallConstructor()
			If Not IsFile(fileId)
				ConstructorFailure
			EndIf
			this\fileId = fileId
		EndConstructor
		
		Deconstructor
		EndDeconstructor
		
		Synchronized(i, close,(this))
			CloseFile(this\fileId)
		EndSynchronized
		
		Synchronized(q, available,(this))
			If Not IsFile(this\fileId)
				SynchronizedReturn(0)
			Else
				SynchronizedReturn(Lof(this\fileId) - Loc(this\fileId))
			EndIf
		EndSynchronized
		
		Synchronized(i, readBuffer,(this, *buffer, size.i))
			If size <= 0
				SynchronizedReturn(0)
			EndIf
			
			If Not IsFile(this\fileId)
				SynchronizedReturn(-1)
			EndIf
			
			Protected available.q = self\available()
			
			If available = 0
				SynchronizedReturn(-1)
			EndIf
			
			If size > available
				size = available
			EndIf
			
			size = ReadData(this\fileId, *buffer, size)
			
			SynchronizedReturn(size)
		EndSynchronized
		
		Synchronized(q, skip,(this, bytes.q))
			If bytes < 0
				SynchronizedReturn(0)
			EndIf
			
			Protected available.q = self\available()
			
			If bytes > available
				bytes = available
			EndIf
			
			FileSeek(this\fileId, bytes, #PB_Relative)
			
			SynchronizedReturn(bytes)
		EndSynchronized
		
		Synchronized(i, mark,(this, readLimit.q))
			this\marked = Loc(this\fileId)
		EndSynchronized
		
		Synchronized(i, markSupported,(this))
			SynchronizedReturn(#True)
		EndSynchronized
		
		Synchronized(i, reset,(this))
			FileSeek(this\fileId, this\marked, #PB_Absolute)
		EndSynchronized
		
		Synchronized(i, getId,(this))
			SynchronizedReturn(this\fileId)
		EndSynchronized
EndClass

CompilerIf #PB_Compiler_IsMainFile
	Define bufferString.s = "%Garbage%This is a test  for the FileInputStream class."
	
	Define *buffer = AllocateMemory(Len(bufferString))
	PokeS(*buffer, bufferString, -1, #PB_Ascii | #PB_String_NoZero)
	
	Define fileName.s = GetTemporaryDirectory() + "FileInputStream.test"
	
	If CreateFile(0, fileName)
		WriteData(0, *buffer, Len(bufferString))
		CloseFile(0)
	EndIf
	
	Define is.type(FileInputStream) = FileInputStream::open(fileName + "a")
	
	Debug "Uses file id: " + is\getId()
	
	; because copyFrom we can free or buffer
	FreeMemory(*buffer)
	
	Debug "Bytes available: " + is\available()
	Debug "Skip 9 bytes..."
	is\skip(9)
	Debug "Bytes available: " + is\available()
	Debug "Output word by word..."
	
	Repeat
		; Mark current position in stream
		is\mark(0)
		
		; Read bytes until a whitespace appears
		Define wordLength.q = 0
		Repeat
			Define c.a, bytesRead.q
			; Be sure to read one byte
			Repeat
				bytesRead = is\readBuffer(@c, 1)
			Until bytesRead <> 0
			wordLength + Bool(c <> ' ')
		Until c = ' ' Or bytesRead = -1
		
		; If word length is zero do not allocate a buffer
		If wordLength = 0
			Debug ""
			Continue
		EndIf
		
		; Reset position instream
		is\reset()
		
		; allocate memory for whole word
		*buffer = AllocateMemory(wordLength)
		
		; read word into buffer
		is\readBuffer(*buffer, wordLength)
		
		; skip whitespace
		is\skip(1)
		
		; output word
		Debug PeekS(*buffer, wordLength, #PB_Ascii)
		
		FreeMemory(*buffer)
		
	Until bytesRead = -1
	
	is\free()
	
CompilerEndIf
; IDE Options = PureBasic 5.40 LTS Beta 9 (Linux - x64)
; CursorPosition = 138
; FirstLine = 115
; Folding = -
; EnableUnicode
; EnableXP
; CompileSourceDirectory
; EnablePurifier